package oclib

import (
	"encoding/json"
	"slices"

	"cloud.o-forge.io/core/oc-lib/dbs"
	"cloud.o-forge.io/core/oc-lib/models/resources"
	"cloud.o-forge.io/core/oc-lib/models/resources/workflow/graph"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/models/workflow_execution"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

type AbstractWorkflow struct {
	resources.ResourceSet
	Graph    *graph.Graph      `bson:"graph,omitempty" json:"graph,omitempty"`
	Schedule *WorkflowSchedule `bson:"schedule,omitempty" json:"schedule,omitempty"`
	Shared   []string          `json:"shared,omitempty" bson:"shared,omitempty"`
}

func (w *AbstractWorkflow) isDCLink(link graph.GraphLink) (bool, string) {
	if slices.Contains(w.Datacenters, link.Source.ID) {
		return true, link.Source.ID
	}
	if slices.Contains(w.Datacenters, link.Destination.ID) {
		return true, link.Destination.ID
	}
	return false, ""
}

type Workflow struct {
	utils.AbstractObject
	AbstractWorkflow
}

func (wfa *Workflow) CheckBooking() (bool, error) {
	// check if
	if wfa.Schedule == nil || wfa.Schedule.Start == nil {
		return false, nil
	}
	if wfa.Schedule.End == nil {
		// if no end... then Book like a savage
		return true, nil
	}
	e := *wfa.Schedule.End
	accessor := wfa.GetAccessor()
	res, code, err := accessor.Search(&dbs.Filters{
		And: map[string][]dbs.Filter{
			"workflowexecution.state": {{Operator: dbs.EQUAL.String(), Value: workflow_execution.SCHEDULED.EnumIndex()}},
			"workflowexecution.execution_date": {
				{Operator: dbs.LTE.String(), Value: primitive.NewDateTimeFromTime(e)},
				{Operator: dbs.GTE.String(), Value: primitive.NewDateTimeFromTime(*wfa.Schedule.Start)},
			},
		},
	}, "")
	if code != 200 {
		return false, err
	}
	return len(res) == 0, nil
}

func (d *Workflow) GetName() string {
	return d.Name
}

func (d *Workflow) GetAccessor() utils.Accessor {
	data := New()
	data.SetLogger(utils.WORKFLOW)
	return data
}

func (dma *Workflow) Deserialize(j map[string]interface{}) utils.DBObject {
	b, err := json.Marshal(j)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, dma)
	return dma
}

func (dma *Workflow) Serialize() map[string]interface{} {
	var m map[string]interface{}
	b, err := json.Marshal(dma)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, &m)
	return m
}
